"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Generator = void 0;
var tslib_1 = require("tslib");
var changeCase = tslib_1.__importStar(require("change-case"));
var dayjs_1 = tslib_1.__importDefault(require("dayjs"));
var fs_extra_1 = tslib_1.__importDefault(require("fs-extra"));
var got_1 = tslib_1.__importDefault(require("got"));
var path_1 = tslib_1.__importDefault(require("path"));
var prettier_1 = tslib_1.__importDefault(require("prettier"));
var vtils_1 = require("vtils");
var utils_1 = require("./utils");
var SwaggerToYApiServer_1 = require("./SwaggerToYApiServer");
var Generator = /** @class */ (function () {
    function Generator(config, options) {
        var _this = this;
        if (options === void 0) { options = { cwd: process.cwd() }; }
        this.options = options;
        /** 配置 */
        this.config = [];
        this.disposes = [];
        this.fetchProject = vtils_1.memoize(function (_a) {
            var serverUrl = _a.serverUrl, token = _a.token;
            return tslib_1.__awaiter(_this, void 0, void 0, function () {
                var projectInfo, basePath;
                return tslib_1.__generator(this, function (_b) {
                    switch (_b.label) {
                        case 0: return [4 /*yield*/, this.fetchApi(serverUrl + "/api/project/get", {
                                token: token,
                            })];
                        case 1:
                            projectInfo = _b.sent();
                            basePath = ("/" + (projectInfo.basepath || '/'))
                                .replace(/\/+$/, '')
                                .replace(/^\/+/, '/');
                            projectInfo.basepath = basePath;
                            return [2 /*return*/, projectInfo];
                    }
                });
            });
        }, function (_a) {
            var serverUrl = _a.serverUrl, token = _a.token;
            return serverUrl + "|" + token;
        });
        this.fetchExport = vtils_1.memoize(function (_a) {
            var serverUrl = _a.serverUrl, token = _a.token;
            return tslib_1.__awaiter(_this, void 0, void 0, function () {
                var projectInfo, categoryList;
                return tslib_1.__generator(this, function (_b) {
                    switch (_b.label) {
                        case 0: return [4 /*yield*/, this.fetchProject({ serverUrl: serverUrl, token: token })];
                        case 1:
                            projectInfo = _b.sent();
                            return [4 /*yield*/, this.fetchApi(serverUrl + "/api/plugin/export", {
                                    type: 'json',
                                    status: 'all',
                                    isWiki: 'false',
                                    token: token,
                                })];
                        case 2:
                            categoryList = _b.sent();
                            return [2 /*return*/, categoryList.map(function (cat) {
                                    cat.list = (cat.list || []).map(function (item) {
                                        item.path = "" + projectInfo.basepath + item.path;
                                        return item;
                                    });
                                    return cat;
                                })];
                    }
                });
            });
        }, function (_a) {
            var serverUrl = _a.serverUrl, token = _a.token;
            return serverUrl + "|" + token;
        });
        // config 可能是对象或数组，统一为数组
        this.config = vtils_1.castArray(config);
    }
    Generator.prototype.prepare = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _a;
            var _this = this;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = this;
                        return [4 /*yield*/, Promise.all(
                            // config 可能是对象或数组，统一为数组
                            this.config.map(function (item) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                                var swaggerToYApiServer_1, _a;
                                return tslib_1.__generator(this, function (_b) {
                                    switch (_b.label) {
                                        case 0:
                                            if (!(item.serverType === 'swagger')) return [3 /*break*/, 2];
                                            swaggerToYApiServer_1 = new SwaggerToYApiServer_1.SwaggerToYApiServer({
                                                swaggerJsonUrl: item.serverUrl,
                                            });
                                            _a = item;
                                            return [4 /*yield*/, swaggerToYApiServer_1.start()];
                                        case 1:
                                            _a.serverUrl = _b.sent();
                                            this.disposes.push(function () { return swaggerToYApiServer_1.stop(); });
                                            _b.label = 2;
                                        case 2:
                                            if (item.serverUrl) {
                                                // 去除地址后面的 /
                                                // fix: https://github.com/fjc0k/yapi-to-typescript/issues/22
                                                item.serverUrl = item.serverUrl.replace(/\/+$/, '');
                                            }
                                            return [2 /*return*/, item];
                                    }
                                });
                            }); }))];
                    case 1:
                        _a.config = _b.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    Generator.prototype.generate = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var outputFileList;
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        outputFileList = Object.create(null);
                        return [4 /*yield*/, Promise.all(this.config.map(function (serverConfig, serverIndex) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                                var projects;
                                var _this = this;
                                return tslib_1.__generator(this, function (_a) {
                                    projects = serverConfig.projects.reduce(function (projects, project) {
                                        projects.push.apply(projects, vtils_1.castArray(project.token).map(function (token) { return (tslib_1.__assign(tslib_1.__assign({}, project), { token: token })); }));
                                        return projects;
                                    }, []);
                                    return [2 /*return*/, Promise.all(projects.map(function (projectConfig, projectIndex) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                                            var projectInfo;
                                            var _this = this;
                                            return tslib_1.__generator(this, function (_a) {
                                                switch (_a.label) {
                                                    case 0: return [4 /*yield*/, this.fetchProjectInfo(tslib_1.__assign(tslib_1.__assign({}, serverConfig), projectConfig))];
                                                    case 1:
                                                        projectInfo = _a.sent();
                                                        return [4 /*yield*/, Promise.all(projectConfig.categories.map(function (categoryConfig, categoryIndex) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                                                                var categoryIds, excludedCategoryIds, codes, _i, _a, groupedCodes;
                                                                var _b;
                                                                var _this = this;
                                                                return tslib_1.__generator(this, function (_c) {
                                                                    switch (_c.label) {
                                                                        case 0:
                                                                            categoryIds = vtils_1.castArray(categoryConfig.id);
                                                                            // 全部分类
                                                                            if (categoryIds.includes(0)) {
                                                                                categoryIds.push.apply(categoryIds, projectInfo.cats.map(function (cat) { return cat._id; }));
                                                                            }
                                                                            // 唯一化
                                                                            categoryIds = vtils_1.uniq(categoryIds);
                                                                            excludedCategoryIds = categoryIds
                                                                                .filter(function (id) { return id < 0; })
                                                                                .map(Math.abs);
                                                                            categoryIds = categoryIds.filter(function (id) { return !excludedCategoryIds.includes(Math.abs(id)); });
                                                                            // 删除不存在的分类
                                                                            categoryIds = categoryIds.filter(function (id) { return !!projectInfo.cats.find(function (cat) { return cat._id === id; }); });
                                                                            // 顺序化
                                                                            categoryIds = categoryIds.sort();
                                                                            return [4 /*yield*/, Promise.all(categoryIds.map(function (id, categoryIndex2) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                                                                                    var syntheticalConfig, interfaceList, outputFilePath, categoryUID, categoryCode, _a, _b;
                                                                                    var _this = this;
                                                                                    return tslib_1.__generator(this, function (_c) {
                                                                                        switch (_c.label) {
                                                                                            case 0:
                                                                                                categoryConfig = tslib_1.__assign(tslib_1.__assign({}, categoryConfig), { id: id });
                                                                                                syntheticalConfig = tslib_1.__assign(tslib_1.__assign(tslib_1.__assign(tslib_1.__assign({}, serverConfig), projectConfig), categoryConfig), { mockUrl: projectInfo.getMockUrl() });
                                                                                                syntheticalConfig.target =
                                                                                                    syntheticalConfig.target || 'typescript';
                                                                                                syntheticalConfig.devUrl = projectInfo.getDevUrl(syntheticalConfig.devEnvName);
                                                                                                syntheticalConfig.prodUrl = projectInfo.getProdUrl(syntheticalConfig.prodEnvName);
                                                                                                return [4 /*yield*/, this.fetchInterfaceList(syntheticalConfig)];
                                                                                            case 1:
                                                                                                interfaceList = (_c.sent())
                                                                                                    .map(function (interfaceInfo) {
                                                                                                    // 实现 _project 字段
                                                                                                    interfaceInfo._project = vtils_1.omit(projectInfo, [
                                                                                                        'cats',
                                                                                                        'getMockUrl',
                                                                                                        'getDevUrl',
                                                                                                        'getProdUrl',
                                                                                                    ]);
                                                                                                    return interfaceInfo;
                                                                                                })
                                                                                                    .sort(function (a, b) { return a._id - b._id; });
                                                                                                outputFilePath = path_1.default.resolve(this.options.cwd, syntheticalConfig.outputFilePath);
                                                                                                categoryUID = "_" + serverIndex + "_" + projectIndex + "_" + categoryIndex + "_" + categoryIndex2;
                                                                                                if (!(interfaceList.length === 0)) return [3 /*break*/, 2];
                                                                                                _a = '';
                                                                                                return [3 /*break*/, 4];
                                                                                            case 2:
                                                                                                _b = [[
                                                                                                        syntheticalConfig.typesOnly
                                                                                                            ? ''
                                                                                                            : vtils_1.dedent(templateObject_1 || (templateObject_1 = tslib_1.__makeTemplateObject(["\n                                  const mockUrl", " = ", " as any\n                                  const devUrl", " = ", " as any\n                                  const prodUrl", " = ", " as any\n                                  const dataKey", " = ", " as any\n                                "], ["\n                                  const mockUrl", " = ",
                                                                                                                " as any\n                                  const devUrl", " = ",
                                                                                                                " as any\n                                  const prodUrl", " = ",
                                                                                                                " as any\n                                  const dataKey", " = ",
                                                                                                                " as any\n                                "])), categoryUID, JSON.stringify(syntheticalConfig.mockUrl), categoryUID, JSON.stringify(syntheticalConfig.devUrl), categoryUID, JSON.stringify(syntheticalConfig.prodUrl), categoryUID, JSON.stringify(syntheticalConfig.dataKey))
                                                                                                    ]];
                                                                                                return [4 /*yield*/, Promise.all(interfaceList.map(function (interfaceInfo) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                                                                                                        return tslib_1.__generator(this, function (_a) {
                                                                                                            interfaceInfo = vtils_1.isFunction(syntheticalConfig.preproccessInterface)
                                                                                                                ? syntheticalConfig.preproccessInterface(interfaceInfo, changeCase)
                                                                                                                : interfaceInfo;
                                                                                                            return [2 /*return*/, this.generateInterfaceCode(syntheticalConfig, interfaceInfo, categoryUID)];
                                                                                                        });
                                                                                                    }); }))];
                                                                                            case 3:
                                                                                                _a = tslib_1.__spreadArrays.apply(void 0, _b.concat([(_c.sent())])).join('\n\n');
                                                                                                _c.label = 4;
                                                                                            case 4:
                                                                                                categoryCode = _a;
                                                                                                if (!outputFileList[outputFilePath]) {
                                                                                                    outputFileList[outputFilePath] = {
                                                                                                        syntheticalConfig: syntheticalConfig,
                                                                                                        content: [],
                                                                                                        requestFunctionFilePath: syntheticalConfig.requestFunctionFilePath
                                                                                                            ? path_1.default.resolve(this.options.cwd, syntheticalConfig.requestFunctionFilePath)
                                                                                                            : path_1.default.join(path_1.default.dirname(outputFilePath), 'request.ts'),
                                                                                                        requestHookMakerFilePath: syntheticalConfig.reactHooks &&
                                                                                                            syntheticalConfig.reactHooks.enabled
                                                                                                            ? syntheticalConfig.reactHooks
                                                                                                                .requestHookMakerFilePath
                                                                                                                ? path_1.default.resolve(this.options.cwd, syntheticalConfig.reactHooks
                                                                                                                    .requestHookMakerFilePath)
                                                                                                                : path_1.default.join(path_1.default.dirname(outputFilePath), 'makeRequestHook.ts')
                                                                                                            : '',
                                                                                                    };
                                                                                                }
                                                                                                return [2 /*return*/, {
                                                                                                        outputFilePath: outputFilePath,
                                                                                                        code: categoryCode,
                                                                                                        weights: [
                                                                                                            serverIndex,
                                                                                                            projectIndex,
                                                                                                            categoryIndex,
                                                                                                            categoryIndex2,
                                                                                                        ],
                                                                                                    }];
                                                                                        }
                                                                                    });
                                                                                }); }))];
                                                                        case 1:
                                                                            codes = _c.sent();
                                                                            for (_i = 0, _a = vtils_1.values(vtils_1.groupBy(codes, function (item) { return item.outputFilePath; })); _i < _a.length; _i++) {
                                                                                groupedCodes = _a[_i];
                                                                                groupedCodes.sort(function (a, b) {
                                                                                    var _a;
                                                                                    var x = a.weights.length > b.weights.length ? b : a;
                                                                                    var minLen = Math.min(a.weights.length, b.weights.length);
                                                                                    var maxLen = Math.max(a.weights.length, b.weights.length);
                                                                                    (_a = x.weights).push.apply(_a, new Array(maxLen - minLen).fill(0));
                                                                                    var w = a.weights.reduce(function (w, _, i) {
                                                                                        if (w === 0) {
                                                                                            w = a.weights[i] - b.weights[i];
                                                                                        }
                                                                                        return w;
                                                                                    }, 0);
                                                                                    return w;
                                                                                });
                                                                                (_b = outputFileList[groupedCodes[0].outputFilePath].content).push.apply(_b, groupedCodes.map(function (item) { return item.code; }));
                                                                            }
                                                                            return [2 /*return*/];
                                                                    }
                                                                });
                                                            }); }))];
                                                    case 2:
                                                        _a.sent();
                                                        return [2 /*return*/];
                                                }
                                            });
                                        }); }))];
                                });
                            }); }))];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, outputFileList];
                }
            });
        });
    };
    Generator.prototype.write = function (outputFileList) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/, Promise.all(Object.keys(outputFileList).map(function (outputFilePath) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                        var _a, 
                        // eslint-disable-next-line prefer-const
                        content, requestFunctionFilePath, requestHookMakerFilePath, 
                        // eslint-disable-next-line prefer-const
                        syntheticalConfig, rawRequestFunctionFilePath, rawRequestHookMakerFilePath, rawOutputContent, prettyOutputContent, outputContent;
                        return tslib_1.__generator(this, function (_b) {
                            switch (_b.label) {
                                case 0:
                                    _a = outputFileList[outputFilePath], content = _a.content, requestFunctionFilePath = _a.requestFunctionFilePath, requestHookMakerFilePath = _a.requestHookMakerFilePath, syntheticalConfig = _a.syntheticalConfig;
                                    rawRequestFunctionFilePath = requestFunctionFilePath;
                                    rawRequestHookMakerFilePath = requestHookMakerFilePath;
                                    // 支持 .jsx? 后缀
                                    outputFilePath = outputFilePath.replace(/\.js(x)?$/, '.ts$1');
                                    requestFunctionFilePath = requestFunctionFilePath.replace(/\.js(x)?$/, '.ts$1');
                                    requestHookMakerFilePath = requestHookMakerFilePath.replace(/\.js(x)?$/, '.ts$1');
                                    if (!!syntheticalConfig.typesOnly) return [3 /*break*/, 3];
                                    return [4 /*yield*/, fs_extra_1.default.pathExists(rawRequestFunctionFilePath)];
                                case 1:
                                    if (!!(_b.sent())) return [3 /*break*/, 3];
                                    return [4 /*yield*/, fs_extra_1.default.outputFile(requestFunctionFilePath, vtils_1.dedent(templateObject_2 || (templateObject_2 = tslib_1.__makeTemplateObject(["\n                import { RequestFunctionParams } from 'yapi-to-typescript'\n\n                export interface RequestOptions {\n                  /**\n                   * \u4F7F\u7528\u7684\u670D\u52A1\u5668\u3002\n                   *\n                   * - `prod`: \u751F\u4EA7\u670D\u52A1\u5668\n                   * - `dev`: \u6D4B\u8BD5\u670D\u52A1\u5668\n                   * - `mock`: \u6A21\u62DF\u670D\u52A1\u5668\n                   *\n                   * @default prod\n                   */\n                  server?: 'prod' | 'dev' | 'mock',\n                }\n\n                export default function request<TResponseData>(\n                  payload: RequestFunctionParams,\n                  options: RequestOptions = {\n                    server: 'prod',\n                  },\n                ): Promise<TResponseData> {\n                  return new Promise<TResponseData>((resolve, reject) => {\n                    // \u57FA\u672C\u5730\u5740\n                    const baseUrl = options.server === 'mock'\n                      ? payload.mockUrl\n                      : options.server === 'dev'\n                        ? payload.devUrl\n                        : payload.prodUrl\n\n                    // \u8BF7\u6C42\u5730\u5740\n                    const url = `${baseUrl}${payload.path}`\n\n                    // \u5177\u4F53\u8BF7\u6C42\u903B\u8F91\n                  })\n                }\n              "], ["\n                import { RequestFunctionParams } from 'yapi-to-typescript'\n\n                export interface RequestOptions {\n                  /**\n                   * \u4F7F\u7528\u7684\u670D\u52A1\u5668\u3002\n                   *\n                   * - \\`prod\\`: \u751F\u4EA7\u670D\u52A1\u5668\n                   * - \\`dev\\`: \u6D4B\u8BD5\u670D\u52A1\u5668\n                   * - \\`mock\\`: \u6A21\u62DF\u670D\u52A1\u5668\n                   *\n                   * @default prod\n                   */\n                  server?: 'prod' | 'dev' | 'mock',\n                }\n\n                export default function request<TResponseData>(\n                  payload: RequestFunctionParams,\n                  options: RequestOptions = {\n                    server: 'prod',\n                  },\n                ): Promise<TResponseData> {\n                  return new Promise<TResponseData>((resolve, reject) => {\n                    // \u57FA\u672C\u5730\u5740\n                    const baseUrl = options.server === 'mock'\n                      ? payload.mockUrl\n                      : options.server === 'dev'\n                        ? payload.devUrl\n                        : payload.prodUrl\n\n                    // \u8BF7\u6C42\u5730\u5740\n                    const url = \\`\\${baseUrl}\\${payload.path}\\`\n\n                    // \u5177\u4F53\u8BF7\u6C42\u903B\u8F91\n                  })\n                }\n              "]))))];
                                case 2:
                                    _b.sent();
                                    _b.label = 3;
                                case 3:
                                    rawOutputContent = vtils_1.dedent(templateObject_3 || (templateObject_3 = tslib_1.__makeTemplateObject(["\n          import { AdminPath, buildApiUrl } from '@/modules/Common/constants/ApiPath';\n          import Res from '@/modules/Common/utils/Res';\n          import { post } from '@/modules/Common/utils/request';\n          ", "\n        "], ["\n          import { AdminPath, buildApiUrl } from '@/modules/Common/constants/ApiPath';\n          import Res from '@/modules/Common/utils/Res';\n          import { post } from '@/modules/Common/utils/request';\n          ", "\n        "
                                        // ref: https://prettier.io/docs/en/options.html
                                    ])), content.join('\n\n').trim());
                                    prettyOutputContent = prettier_1.default.format(rawOutputContent, {
                                        parser: 'typescript',
                                        printWidth: 120,
                                        tabWidth: 2,
                                        singleQuote: true,
                                        semi: false,
                                        trailingComma: 'all',
                                        bracketSpacing: false,
                                        endOfLine: 'lf',
                                    });
                                    outputContent = vtils_1.dedent(templateObject_4 || (templateObject_4 = tslib_1.__makeTemplateObject(["\n          /* prettier-ignore-start */\n          ", "\n          /* prettier-ignore-end */\n        "], ["\n          /* prettier-ignore-start */\n          ", "\n          /* prettier-ignore-end */\n        "])), prettyOutputContent) + "\n";
                                    return [4 /*yield*/, fs_extra_1.default.outputFile(outputFilePath, outputContent)];
                                case 4:
                                    _b.sent();
                                    return [2 /*return*/];
                            }
                        });
                    }); }))];
            });
        });
    };
    Generator.prototype.fetchApi = function (url, query) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var res;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, got_1.default.get(url, {
                            searchParams: query,
                            responseType: 'json',
                        })
                        /* istanbul ignore next */
                    ];
                    case 1:
                        res = (_a.sent()).body;
                        /* istanbul ignore next */
                        if (res && res.errcode) {
                            utils_1.throwError(res.errmsg);
                        }
                        return [2 /*return*/, res.data || res];
                }
            });
        });
    };
    /** 获取分类的接口列表 */
    Generator.prototype.fetchInterfaceList = function (_a) {
        var serverUrl = _a.serverUrl, token = _a.token, id = _a.id;
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var category;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0: return [4 /*yield*/, this.fetchExport({ serverUrl: serverUrl, token: token })];
                    case 1:
                        category = ((_b.sent()) || []).find(function (cat) { return !vtils_1.isEmpty(cat) && !vtils_1.isEmpty(cat.list) && cat.list[0].catid === id; });
                        if (category) {
                            category.list.forEach(function (interfaceInfo) {
                                // 实现 _category 字段
                                interfaceInfo._category = vtils_1.omit(category, ['list']);
                            });
                        }
                        return [2 /*return*/, category ? category.list : []];
                }
            });
        });
    };
    /** 获取项目信息 */
    Generator.prototype.fetchProjectInfo = function (syntheticalConfig) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var projectInfo, projectCats;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.fetchProject(syntheticalConfig)];
                    case 1:
                        projectInfo = _a.sent();
                        return [4 /*yield*/, this.fetchApi(syntheticalConfig.serverUrl + "/api/interface/getCatMenu", {
                                token: syntheticalConfig.token,
                                project_id: projectInfo._id,
                            })];
                    case 2:
                        projectCats = _a.sent();
                        return [2 /*return*/, tslib_1.__assign(tslib_1.__assign({}, projectInfo), { cats: projectCats, getMockUrl: function () {
                                    return syntheticalConfig.serverUrl + "/mock/" + projectInfo._id;
                                }, getDevUrl: function (devEnvName) {
                                    var env = projectInfo.env.find(function (e) { return e.name === devEnvName; });
                                    return (env && env.domain) /* istanbul ignore next */ || '';
                                }, getProdUrl: function (prodEnvName) {
                                    var env = projectInfo.env.find(function (e) { return e.name === prodEnvName; });
                                    return (env && env.domain) /* istanbul ignore next */ || '';
                                } })];
                }
            });
        });
    };
    /** 生成接口代码 */
    Generator.prototype.generateInterfaceCode = function (syntheticalConfig, interfaceInfo, categoryUID) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var extendedInterfaceInfo, requestFunctionName, _a, requestConfigName, requestConfigTypeName, requestDataTypeName, _b, responseDataTypeName, _c, requestDataJsonSchema, requestDataType, responseDataJsonSchema, responseDataType, escapedTitle, interfaceTitle, interfaceSummary, interfaceExtraComments;
            return tslib_1.__generator(this, function (_d) {
                switch (_d.label) {
                    case 0:
                        extendedInterfaceInfo = tslib_1.__assign(tslib_1.__assign({}, interfaceInfo), { parsedPath: path_1.default.parse(interfaceInfo.path) });
                        if (!vtils_1.isFunction(syntheticalConfig.getRequestFunctionName)) return [3 /*break*/, 2];
                        return [4 /*yield*/, syntheticalConfig.getRequestFunctionName(extendedInterfaceInfo, changeCase)];
                    case 1:
                        _a = _d.sent();
                        return [3 /*break*/, 3];
                    case 2:
                        _a = changeCase.camelCase(extendedInterfaceInfo.parsedPath.name);
                        _d.label = 3;
                    case 3:
                        requestFunctionName = _a;
                        requestConfigName = changeCase.camelCase(requestFunctionName + "RequestConfig");
                        requestConfigTypeName = changeCase.pascalCase(requestConfigName);
                        if (!vtils_1.isFunction(syntheticalConfig.getRequestDataTypeName)) return [3 /*break*/, 5];
                        return [4 /*yield*/, syntheticalConfig.getRequestDataTypeName(extendedInterfaceInfo, changeCase)];
                    case 4:
                        _b = _d.sent();
                        return [3 /*break*/, 6];
                    case 5:
                        _b = changeCase.pascalCase(requestFunctionName + "Request");
                        _d.label = 6;
                    case 6:
                        requestDataTypeName = _b;
                        if (!vtils_1.isFunction(syntheticalConfig.getResponseDataTypeName)) return [3 /*break*/, 8];
                        return [4 /*yield*/, syntheticalConfig.getResponseDataTypeName(extendedInterfaceInfo, changeCase)];
                    case 7:
                        _c = _d.sent();
                        return [3 /*break*/, 9];
                    case 8:
                        _c = changeCase.pascalCase(requestFunctionName + "Response");
                        _d.label = 9;
                    case 9:
                        responseDataTypeName = _c;
                        requestDataJsonSchema = utils_1.getRequestDataJsonSchema(extendedInterfaceInfo);
                        return [4 /*yield*/, utils_1.jsonSchemaToType(requestDataJsonSchema, requestDataTypeName)];
                    case 10:
                        requestDataType = _d.sent();
                        responseDataJsonSchema = utils_1.getResponseDataJsonSchema(extendedInterfaceInfo, syntheticalConfig.dataKey);
                        return [4 /*yield*/, utils_1.jsonSchemaToType(responseDataJsonSchema, responseDataTypeName)
                            // 转义标题中的 /
                        ];
                    case 11:
                        responseDataType = _d.sent();
                        escapedTitle = String(extendedInterfaceInfo.title).replace(/\//g, '\\/');
                        interfaceTitle = "[" + escapedTitle + "\u2197](" + syntheticalConfig.serverUrl + "/project/" + extendedInterfaceInfo.project_id + "/interface/api/" + extendedInterfaceInfo._id + ")";
                        interfaceSummary = [
                            {
                                label: '分类',
                                value: "[" + extendedInterfaceInfo._category.name + "\u2197](" + syntheticalConfig.serverUrl + "/project/" + extendedInterfaceInfo.project_id + "/interface/api/cat_" + extendedInterfaceInfo.catid + "/" + extendedInterfaceInfo.desc + ")",
                            },
                            {
                                label: '标签',
                                value: extendedInterfaceInfo.tag.map(function (tag) { return "`" + tag + "`"; }),
                            },
                            {
                                label: '请求头',
                                value: "`" + extendedInterfaceInfo.method.toUpperCase() + " " + extendedInterfaceInfo.path + "`",
                            },
                            {
                                label: '更新时间',
                                value: process.env.JEST_WORKER_ID // 测试时使用 unix 时间戳
                                    ? String(extendedInterfaceInfo.up_time)
                                    : /* istanbul ignore next */
                                        "`" + dayjs_1.default(extendedInterfaceInfo.up_time * 1000).format('YYYY-MM-DD HH:mm:ss') + "`",
                            },
                        ];
                        interfaceExtraComments = interfaceSummary
                            .filter(function (item) { return !vtils_1.isEmpty(item.value); })
                            .map(function (item) { return "* @" + item.label + " " + vtils_1.castArray(item.value).join(', '); })
                            .join('\n');
                        return [2 /*return*/, vtils_1.dedent(templateObject_6 || (templateObject_6 = tslib_1.__makeTemplateObject(["\n      ", "\n\n      ", "\n\n      ", "\n    "], ["\n      ", "\n\n      ", "\n\n      ",
                                "\n    "])), requestDataType.trim(), responseDataType.trim(), vtils_1.dedent(templateObject_5 || (templateObject_5 = tslib_1.__makeTemplateObject(["\n          /**\n           * \u63A5\u53E3 ", " \u7684 **\u8BF7\u6C42\u51FD\u6570**\n           *\n           ", "\n          */\n          function ", "(body: ", ") {\n            return ", "<", ">(buildUrl('", "'), body);\n          }\n        "], ["\n          /**\n           * \u63A5\u53E3 ", " \u7684 **\u8BF7\u6C42\u51FD\u6570**\n           *\n           ", "\n          */\n          function ", "(body: ", ") {\n            return ", "<", ">(buildUrl('", "'), body);\n          }\n        "])), interfaceTitle, interfaceExtraComments, requestFunctionName, requestDataTypeName, extendedInterfaceInfo.method.toLowerCase(), responseDataTypeName, extendedInterfaceInfo.path))];
                }
            });
        });
    };
    Generator.prototype.destroy = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/, Promise.all(this.disposes.map(function (dispose) { return tslib_1.__awaiter(_this, void 0, void 0, function () { return tslib_1.__generator(this, function (_a) {
                        return [2 /*return*/, dispose()];
                    }); }); }))];
            });
        });
    };
    return Generator;
}());
exports.Generator = Generator;
var templateObject_1, templateObject_2, templateObject_3, templateObject_4, templateObject_5, templateObject_6;
